package com.jtauber.fop.apps;

// SAX
import org.xml.sax.Parser;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

// Java
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.net.URL;

/**
 * mainline class
 *
 * gets input and output filenames from the command line
 * creates a SAX Parser (defaulting to XP)
 * 
 */
public class CommandLine {

/**
 * creates a SAX parser, using the value of org.xml.sax.parser defaulting to com.jclark.xml.sax.Driver
 */
static Parser createParser() {
	// could use org.xml.sax.helpers.ParserFactory, but do it myself like XT
	
	String parserClassName = System.getProperty("org.xml.sax.parser");
	if (parserClassName == null)
		parserClassName = "com.jclark.xml.sax.Driver";
	System.err.println("using SAX parser " + parserClassName);

	try {
		return (Parser)Class.forName(parserClassName).newInstance();
	} catch (ClassNotFoundException e) {
		System.err.println("Could not find " + parserClassName);
	} catch (InstantiationException e) {
		System.err.println("Could not instantiate " + parserClassName);
	} catch (IllegalAccessException e) {
		System.err.println("Could not access " + parserClassName);
	} catch (ClassCastException e) {
		System.err.println(parserClassName + " is not a SAX driver");
	}
	return null;
}
/**
 * mainline method
 *
 * first command line argument is input file
 * second command line argument is output file
 */
public static void main(String[] args) {
		System.err.println(com.jtauber.fop.Version.getVersion());
		
		if (args.length != 2) {
			System.err.println("usage: java com.jtauber.fop.apps.CommandLine formatting-object-file pdf-file");
			System.exit(1);
		}
		
		Parser parser = createParser();
		
		if (parser == null) {
			System.err.println("ERROR: Unable to create SAX parser");
			System.exit(1);
		}
		
		try {
			PrintWriter writer = new PrintWriter(new FileWriter(args[1]));
			SAXProcessor fop = new SAXProcessor(parser, new InputSource(new FileReader(args[0])));
			fop.format(writer);
		} catch (FOPException e) {
			System.err.println("FATAL ERROR: "+ e.getMessage());
			System.exit(1);
		} catch (FileNotFoundException e) {
			System.err.println("FATAL ERROR: File not found: "+args[0]);
			System.exit(1);
		} catch (IOException e) {
			System.err.println("FATAL ERROR: I/O problem with "+args[1]);
			System.exit(1);
		}
	}
}