package com.jtauber.fop.apps;

// FOP
import com.jtauber.fop.fo.FOTreeBuilder;

// DOM
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;

// SAX
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributeListImpl;

// Java
import java.io.PrintWriter;

/**
 * A FOProcessor that takes a DOM Document, walks the tree, firing events as if a SAX parser
 *
 * Instantiate this class with a DOM Document and then run the format method with a PrintWriter the PDF is to be given to
 *
 * based on John Cowan's work
 */
public class DOMProcessor extends FOProcessor 
{
	private Document document; // the DOM document
/**
 * create a DOM-driven FOProcessor with the given DOM Document
 */
public DOMProcessor(Document document) {
	super(); // set up the FOProcessor
	this.document = document;
}
/**
 * Format the document to PDF on the given PrintWriter
 *
 * Most of the ideas in this method come from John Cowan
 */
public void format(PrintWriter writer) throws FOPException {
	Node currentNode;
	AttributeListImpl currentAtts;
	
	char[] array = null; // temporary array for making Strings into character arrays
	currentAtts = new AttributeListImpl();

	currentNode = this.document; // start at the document element

	try {
		while (currentNode != null) {
			switch (currentNode.getNodeType()) {
				case Node.DOCUMENT_NODE:
			        this.treeBuilder.startDocument();
					break;
				case Node.CDATA_SECTION_NODE:
				case Node.TEXT_NODE:
					String data = currentNode.getNodeValue();
					int datalen = data.length();
					if (array == null || array.length < datalen) // if the array isn't big enough
						array = new char[datalen];               // make a new one
					data.getChars(0, datalen, array, 0);
					this.treeBuilder.characters(array, 0, datalen);
					break;
				case Node.PROCESSING_INSTRUCTION_NODE:
					this.treeBuilder.processingInstruction(currentNode.getNodeName(),currentNode.getNodeValue());
					break;
				case Node.ELEMENT_NODE:
					NamedNodeMap map = currentNode.getAttributes();
					currentAtts.clear();
					for (int i = map.getLength() - 1; i >= 0; i--) {
						Attr att = (Attr)(map.item(i));
						currentAtts.addAttribute(att.getName(), "CDATA", att.getValue());
					}
					this.treeBuilder.startElement(currentNode.getNodeName(), currentAtts);
					break;
			}

			Node nextNode = currentNode.getFirstChild();
			if (nextNode != null) {
				currentNode = nextNode;
				continue;
			}

			while (currentNode != null) {
				switch (currentNode.getNodeType()) {
					case Node.DOCUMENT_NODE:
						this.treeBuilder.endDocument();
						break;
					case Node.ELEMENT_NODE:
						this.treeBuilder.endElement(currentNode.getNodeName());
						break;
				}

				nextNode = currentNode.getNextSibling();
				if (nextNode != null) {
					currentNode = nextNode;
					break;
				}

				currentNode = currentNode.getParentNode();
			}
		}
	} catch (SAXException e) {
		throw new FOPException(e.getMessage());
	}
	layoutput(writer);
}
}