package com.jtauber.fop.apps;

// FOP
import com.jtauber.fop.layout.Document;
import com.jtauber.fop.layout.Layout;
import com.jtauber.fop.layout.pdf.PDFLayout;
import com.jtauber.fop.layout.XMLLayout;
import com.jtauber.fop.fo.FOTreeBuilder;

// Java
import java.io.PrintWriter;
import java.io.IOException;

/**
 * base class for FOP Processors
 *
 * should be subclassed for each way of acquiring a formatting object tree
 * format(PrintWriter) should trigger the methods of FOTreeBuilder then run
 * layoutput(PrintWriter) (implemented in this class)
 */
public abstract class FOProcessor {
	private Layout layout;
	private Document doc;
	protected FOTreeBuilder treeBuilder;

/**
 * create a new FOP Processor
 */
protected FOProcessor() {
	this.layout = new PDFLayout(com.jtauber.fop.Version.getVersion()); // select PDF Layout
	this.doc = new Document(this.layout);
	this.treeBuilder = new FOTreeBuilder(this.doc);
}
/**
 * this method, when implemented, should trigger the methods of FOTreeBuilder then run
 * layoutput(PrintWriter) (implemented in this class)
 */
public abstract void format(PrintWriter writer) throws FOPException;
/**
 * lays out the document and outputs the PDF to the given PrintWriter
 *
 * generally called by a subclasses's format(PrintWriter) method
 */
protected final void layoutput(PrintWriter writer) throws FOPException {
	this.treeBuilder.go();
		
	try {
		this.layout.go(this.doc,writer);
	} catch (IOException e) {
		throw new FOPException(e.getMessage());
	}
}
}