package com.jtauber.fop.layout.pdf;

import com.jtauber.fop.layout.pdf.fonts.*;
import com.jtauber.fop.layout.FontInfo;
import com.jtauber.pdf.PDFDocument;
import com.jtauber.pdf.PDFResources;
import java.util.Enumeration;
import java.util.Hashtable;

public class FontSetup {

	public static void addToResources(PDFDocument doc, FontInfo fontInfo) {
		/* use the given PDFDocument to make the fonts and add them to the
		given resource object */

		Hashtable fonts = fontInfo.getFonts();
		Enumeration e = fonts.keys();
		PDFResources resources = doc.getResources();
		while (e.hasMoreElements()) {
			String f = (String)e.nextElement();
			resources.addFont(
				doc.makeFont(
					f,
					((Font)fonts.get(f)).fontName(),
					((Font)fonts.get(f)).encoding()
				)
			);
		}
	}
	public static void setup(FontInfo fontInfo) {
		/* called to add metrics for basic fonts and useful
		family-style-weight triplets for lookup */

		System.err.println("setting up fonts");
		
		fontInfo.addMetrics("F1",new Helvetica());
		fontInfo.addMetrics("F2",new HelveticaOblique());
		fontInfo.addMetrics("F3",new HelveticaBold());
		fontInfo.addMetrics("F4",new HelveticaBoldOblique());
		fontInfo.addMetrics("F5",new TimesRoman());
		fontInfo.addMetrics("F6",new TimesItalic());
		fontInfo.addMetrics("F7",new TimesBold());
		fontInfo.addMetrics("F8",new TimesBoldItalic());
		fontInfo.addMetrics("F9",new Courier());
		fontInfo.addMetrics("F10",new CourierOblique());
		fontInfo.addMetrics("F11",new CourierBold());
		fontInfo.addMetrics("F12",new CourierBoldOblique());
		
		// any is treated as serif
		fontInfo.addFontProperties("F5","any","normal","normal");
		fontInfo.addFontProperties("F6","any","italic","normal");
		fontInfo.addFontProperties("F6","any","oblique","normal");
		fontInfo.addFontProperties("F7","any","normal","bold");
		fontInfo.addFontProperties("F8","any","italic","bold");
		fontInfo.addFontProperties("F8","any","oblique","bold");
		
		fontInfo.addFontProperties("F1","sans-serif","normal","normal");
		fontInfo.addFontProperties("F2","sans-serif","oblique","normal");
		fontInfo.addFontProperties("F2","sans-serif","italic","normal");
		fontInfo.addFontProperties("F3","sans-serif","normal","bold");
		fontInfo.addFontProperties("F4","sans-serif","oblique","bold");
		fontInfo.addFontProperties("F4","sans-serif","italic","bold");
		fontInfo.addFontProperties("F5","serif","normal","normal");
		fontInfo.addFontProperties("F6","serif","oblique","normal");
		fontInfo.addFontProperties("F6","serif","italic","normal");
		fontInfo.addFontProperties("F7","serif","normal","bold");
		fontInfo.addFontProperties("F8","serif","oblique","bold");
		fontInfo.addFontProperties("F8","serif","italic","bold");
		fontInfo.addFontProperties("F9","monospace","normal","normal");
		fontInfo.addFontProperties("F10","monospace","oblique","normal");
		fontInfo.addFontProperties("F10","monospace","italic","normal");
		fontInfo.addFontProperties("F11","monospace","normal","bold");
		fontInfo.addFontProperties("F12","monospace","oblique","bold");
		fontInfo.addFontProperties("F12","monospace","italic","bold");
	}
}